<?php

/**
 * If called from a cache, Drupal wasn't bootstrapped and the following aren't
 * defined, so we define them.
 */
if (!defined('AD_GEOIP_ALL')) {
  define('AD_GEOIP_ALL', 0);
  define('AD_GEOIP_SELECTED', 1);
  define('AD_GEOIP_NOT_SELECTED', 2);
}

/**
 * Select a geoip appropriate advertisement to display, from database.
 */
function ad_geoip_adserve($ads, $quantity) {
  _debug_echo('Entered function ad_geoip_adserve().');

  $valid = array();
  foreach ($ads as $aid) {
    _debug_echo("ad_geoip: processing aid $aid.");
    $display = TRUE;
    $result = db_query('SELECT code, format FROM {ad_geoip_ads} WHERE aid = %d', $aid);
    while ($geoip = db_fetch_object($result)) {
      if ($geoip->format == AD_GEOIP_ALL) {
        // display ad to all countries
        _debug_echo("ad_geoip: ad valid in all countries.");
        $display = TRUE;
        break;
      }
      else if ($geoip->format == AD_GEOIP_SELECTED) {
        // only display ad to not selected countries
        _debug_echo("ad_geoip: only selected countries are valid.");
        if ($geoip->code == ad_geoip_code()) {
          _debug_echo("ad_geoip: current visitor from $geoip->code, display ad.");
          $display = TRUE;
          break;
        }
        else {
          _debug_echo('ad_geoip: current visitor from '. ad_geoip_code() .", not from $geoip->code.");
          $display = FALSE;
        }
      }
      else if ($geoip->format == AD_GEOIP_NOT_SELECTED) {
        // only display ad to not selected countries
        _debug_echo("ad_geoip: only not selected countries are valid.");
        if ($geoip->code == ad_geoip_code()) {
          _debug_echo("ad_geoip: current visitor from $geoip->code, do not display ad.");
          $display = FALSE;
          break;
        }
      }
    }
    if ($display) {
      $valid[] = $aid;
    }
  }

  $total = sizeof($valid);
  if ($total) {
    $id = $total > 1 ? $valid[mt_rand(0, $total - 1)] : $valid[0];
    _debug_echo("ad_geoip: randomly selected ID: $id.");
    return $id;
  }
  else {
    return (-1);
  }
}

/**
 * Select a geoip appropriate advertisement to display, from cache.
 * TODO: Make function and remove duplicated code.
 */
function ad_geoip_cache_select($ads, $invalid, $cache) {
  _debug_echo("ad_geoip_cache: adserve_select");
  $valid = array();
  foreach ($ads as $aid) {
    if (in_array($aid, $invalid)) {
      _debug_echo("ad_geoip_cache: aid($aid) is invalid.");
    }
    $format = $cache['geoip']['format'][$aid] ? $cache['geoip']['format'][$aid] : AD_GEOIP_ALL;
    _debug_echo("ad_geoip_cache: aid($aid) format($format)");
    $display = TRUE;

    foreach ($cache['geoip'][$aid] as $code) {
      if ($format == AD_GEOIP_ALL) {
        // display ad to all countries
        _debug_echo("ad_geoip_cache: ad valid in all countries.");
        $display = TRUE;
        break;
      }
      else if ($format == AD_GEOIP_SELECTED) {
        // only display ad to not selected countries
        _debug_echo("ad_geoip_cache: only selected countries are valid.");
        if ($code == ad_geoip_code()) {
          _debug_echo("ad_geoip_cache: current visitor from $code, display ad.");
          $display = TRUE;
          break;
        }
        else if (ad_geoip_code() == 'UNKNOWN') {
          // no need to keep check against all possibilities, will never match
          _debug_echo('ad_geoip_cache: current visitor from UNKOWN locale, do not display.');
          $display = FALSE;
          break;
        }
        else {
          _debug_echo('ad_geoip_cache: current visitor from '. ad_geoip_code() .", not from $code.");
          $display = FALSE;
        }
      }
      else if ($format == AD_GEOIP_NOT_SELECTED) {
        // only display ad to not selected countries
        _debug_echo("ad_geoip_cache: only not selected countries are valid, testing $code.");
        if ($code == ad_geoip_code()) {
          _debug_echo("ad_geoip_cache: current visitor from $geoip->code, do not display ad.");
          $display = FALSE;
          break;
        }
      }
    }

    if ($display) {
      _debug_echo("ad_geoip_cache: aid($aid) is valid");
      $valid[] = $aid;
    }
    else {
      _debug_echo("ad_geoip_cache: aid($aid) is not valid");
    }
  }

  $total = sizeof($valid);
  _debug_echo("ad_geoip_cache: selecting from $total ads.");
  if ($total) {
    $id = $total > 1 ? $valid[mt_rand(0, $total - 1)] : $valid[0];
    _debug_echo("ad_geoip_cache: randomly selected ID: $id.");
    return array($id);
  }
  else {
    // no more ads available
    return array(-1);
  }
}

/**
 * Retrieve country code from IP address, cache for re-use.
 */
function ad_geoip_code() {
  static $code = NULL;

  if (is_null($code)) {
    $hostname = $_SERVER['REMOTE_ADDR'];
    $code = geoip_country_code_by_name($hostname);
    if (!$code) {
      _debug_echo("ad_geoip: no code found for host from $hostname.");
      // do not translate, as we may call this without bootstrapping Drupal
      $code = 'UNKNOWN';
    }
  }
  return $code;
}

